;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x561b0fbf221efdf9fac527b0ea135f21 . "./code.cg2js/cg-acache.fasl")
(#x0646014b6f78104f86463fe6fd7ff2af . "./code.cg2js/cg-base.fasl")
(#x7786bdf8d5831db5dbfcdad18ecac5ed . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xcc0a960b6a6b7216c57db5158ea2f9e5 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x4fa297f4ee3c7395bcaec74a37d20156 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x9f46ff482b14274028fb4e71f1b80dc9 . "./code.cg2js/cg-button.fasl")
(#x5773eefaa76bb86737de8560a4dbe5bf . "./code.cg2js/cg-calendar.fasl")
(#xb6c436df926623a35183f812b384c691 . "./code.cg2js/cg-caret.fasl")
(#x0452537cdc5682927f4ae40dff83685a . "./code.cg2js/cg-chart-or-plot.fasl")
(#x0b1e3640f8f67e650cc3a1289d3e6fbf . "./code.cg2js/cg-chart-widget.fasl")
(#x7dd79dafe3b0b17a7e3ebf72a7c41e35 . "./code.cg2js/cg-check-box.fasl")
(#x2aab6987bc342bb02d90149d106491a9 . "./code.cg2js/cg-choice-list.fasl")
(#x75215e22cbee14201d9dd938e46592a7 . "./code.cg2js/cg-choose-printer.fasl")
(#x71a968578b543d0a63be8fa59f795d10 . "./code.cg2js/cg-class-grid.fasl")
(#x0d5ad8ea6e474c1c3f376ad98e642e1d . "./code.cg2js/cg-class-slot-grid.fasl")
(#xb6653f46e57dffc3bdbaee32893191a3 . "./code.cg2js/cg-class-support.fasl")
(#xabc2d6d31fa5bc2c1ce50239ae150db3 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x2f4c617cc279b6fd938867842aa93adb . "./code.cg2js/cg-clipboard-stack.fasl")
(#x82dcf01f19aa58937b100b5ba307942c . "./code.cg2js/cg-clipboard.fasl")
(#xfec76a23f901fd212a2df1757cdecda8 . "./code.cg2js/cg-color-dialog.fasl")
(#x00f29a950355478d42a464bd025ed2d5 . "./code.cg2js/cg-combo-box.fasl")
(#x17ec6304382a8efaaf410d94f6061c12 . "./code.cg2js/cg-common-control.fasl")
(#xe5e5f2e2e8b9c8171a7b1009defdb5d8 . "./code.cg2js/cg-comtab.fasl")
(#x24c92299aff7d4161495eb75b94f4c3a . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xc79f954b922d83f0c5f3a5fe90f7f652 . "./code.cg2js/cg-curve.fasl")
(#x738d4bf9898aceabb9bf7f4c5f6accc7 . "./code.cg2js/cg-dde-utils.fasl")
(#xdb17d824afc746b19526fc4a603dcd02 . "./code.cg2js/cg-dev.fasl")
(#x43e0be06c889057b5e97c57e1308c434 . "./code.cg2js/cg-dialog-item.fasl")
(#xbc87b272758cba746f3f9b1b99beef74 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x33df7f50a35075829e75af14393eb3cb . "./code.cg2js/cg-directory-dialog.fasl")
(#x9677f8f8a042084fafd85993e9c3781a . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xbd86afeddc252b4dfeb147acdb53b57b . "./code.cg2js/cg-drag-and-drop.fasl")
(#x97bccf089c5a2b66f77f38a9bb3516bb . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x8d9f4a6b51519c81a9fa1f2554e7898c . "./code.cg2js/cg-drawable.fasl")
(#x25e554cf51602071aeb1c35cf3c1db9a . "./code.cg2js/cg-dropping-outline.fasl")
(#xb02bf15e2db33f2e83a575c0a7f081ce . "./code.cg2js/cg-edit-in-place.fasl")
(#xafffc50389bbdad46e3ec3f31f4f0e72 . "./code.cg2js/cg-editable-text.fasl")
(#x7b41dd28461cc226851e3ce356fe48f6 . "./code.cg2js/cg-file-dialog.fasl")
(#xb36d16dd5e45fdc980fadb586f93e60c . "./code.cg2js/cg-fill-texture.fasl")
(#x3f600bac69c6151f72c3756b4c108246 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x16eb575db0cef20bb6b2682b79141abe . "./code.cg2js/cg-font-dialog.fasl")
(#xed5fe24d5c6b02aae54bb477ca0486fb . "./code.cg2js/cg-gesture-emulation.fasl")
(#x2faebffc00614d5f7e4258176d316e13 . "./code.cg2js/cg-get-pixmap.fasl")
(#x998a11a85d88482ae3b4201c08ba9121 . "./code.cg2js/cg-get-position.fasl")
(#x245904b49252d5758fd7cfa6feb8720f . "./code.cg2js/cg-graphics-context.fasl")
(#x6a18564bb8e5c0b130aed186205195cc . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x32a8f6a0d15624a48dec571547efb54d . "./code.cg2js/cg-grid-widget.fasl")
(#xd76ea76f08352d879702b4fbd0fa496c . "./code.cg2js/cg-group-box.fasl")
(#x9c4db2665922faf53c9f01fd251d0a1b . "./code.cg2js/cg-header-control.fasl")
(#xc66c608529f39b3b4ea97d2d6b9403dd . "./code.cg2js/cg-hotspot.fasl")
(#x3d991b1c0f8115771eadf6f07fb545b6 . "./code.cg2js/cg-html-dialog.fasl")
(#x0a826ec9cf1fa267a72e8bbed4de92a4 . "./code.cg2js/cg-html-widget.fasl")
(#x8c56cd7985b5a95ed96967fae63947fc . "./code.cg2js/cg-icon-pixmap.fasl")
(#xacec07e1a5c7ab9566684e7ecc09c85d . "./code.cg2js/cg-icon.fasl")
(#x86669074e168338dc468aabdc1e70b45 . "./code.cg2js/cg-ie.fasl")
(#x6018c3cb2dc5ac3c941fa4287b94c1f3 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe0303a4bb8c98ceec44d3a1ecfc74c3b . "./code.cg2js/cg-intersect.fasl")
(#x6a27e51430879fa774c86c8fea118f49 . "./code.cg2js/cg-item-list.fasl")
(#xeb3414b563f8d79b4ab5ee0452f96172 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xcc340debe29694d344f435aa0186f7fe . "./code.cg2js/cg-lamp.fasl")
(#xb130b4431e3e4f38da0f29f1f13b0570 . "./code.cg2js/cg-layout.fasl")
(#x2e68617f37fa85045c997b5f0d4dc482 . "./code.cg2js/cg-lettered-menu.fasl")
(#x912b559e3626090718a17a003e3f5c63 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x18b79a90740d0b3a8b4fb70d027f5da9 . "./code.cg2js/cg-lisp-text.fasl")
(#x6c5a1abc4cd076067c65fcc3e04cf157 . "./code.cg2js/cg-lisp-widget.fasl")
(#xec92f2639a33c7a704895c41219e285f . "./code.cg2js/cg-list-view.fasl")
(#xd635c2c63d275f05191679c900d2e4bc . "./code.cg2js/cg-mci.fasl")
(#xfb0131308b6113c9b66c7ae4197e00ef . "./code.cg2js/cg-menu-tooltip.fasl")
(#x31c48f63434c427a7c57e93ac7259bed . "./code.cg2js/cg-menu.fasl")
(#x4cc6f214b22ac435553cb6b890a17cfd . "./code.cg2js/cg-message-dialog.fasl")
(#x42f16c12ab395f219847228e6b65a3e4 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x5231686d240d7fb7aea540f198775f51 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x1da8419c4654ee11e39e678477db12c0 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x13e917061fcb8de0dcdc0b92a8674bb8 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf88d297703bb0c7438151b502cdabc5c . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe552f3b47d5fba29fc615ccfe33816ad . "./code.cg2js/cg-nodes.fasl")
(#xa06f850795a7bc08d129518ddf78f646 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x407c06cd5f2bec7f834d4b1a81088855 . "./code.cg2js/cg-object-editor.fasl")
(#xc0e5ef7f49bc175d6f0ae9f6d86dd4d5 . "./code.cg2js/cg-ocx.fasl")
(#xe7dffeda539ab9d5ccd188849420ad3b . "./code.cg2js/cg-os-widget.fasl")
(#x8cd8634b182778645c81593f060f75a3 . "./code.cg2js/cg-os-window.fasl")
(#x2b72be0ed2a99d7af6571d85e8f1d853 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xf0b2d9e8a43634e0fe473aab07a15f53 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x84382d173d2add1c2ed218e4a1964253 . "./code.cg2js/cg-outline.fasl")
(#x7f748fd2fb112086fb9660802ad56722 . "./code.cg2js/cg-palette.fasl")
(#xcc561fed2a79fffb9034b388737c5347 . "./code.cg2js/cg-paren-matching.fasl")
(#xb3ea04d26382b29ee98941f862220cb0 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x679df9d9e2dbde6feaa5711f77dcac79 . "./code.cg2js/cg-picture-widget.fasl")
(#x10207abc48ccb05ed45e82c156bfe746 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x25b599a3b480a74f9e68004ce74daa14 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x8ab159aee1c33b6a09c92e2c1bdafcd9 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x474710c6faea757eaf25b0e1506e5bc9 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3be7382aea2f034d2d42b5ad10b13b02 . "./code.cg2js/cg-pixmap.fasl")
(#x69c482fa8d4473285496dc738b5abcbe . "./code.cg2js/cg-plot-widget.fasl")
(#xb92abf355bf616c1d81cc7cb92166f85 . "./code.cg2js/cg-printing.fasl")
(#xe64a48af03b64687485e220dcc96e6f4 . "./code.cg2js/cg-progress-indicator.fasl")
(#x4f445acd3a4e3c4294d673a73291f483 . "./code.cg2js/cg-project-window.fasl")
(#xf789baa6479a69e1d3241a96509c9e3c . "./code.cg2js/cg-property.fasl")
(#xf9617d36b38f63f27108a3cda52ed510 . "./code.cg2js/cg-radio-button.fasl")
(#x2160c30d5addf357e45cd616803c0db9 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x860567278bf8b4c684acedf31f0a221c . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xf0241a9563e5037f87f2ae69d11d5365 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xa60ace1018adf6d1045b0039f7ab5851 . "./code.cg2js/cg-rich-edit.fasl")
(#x0fc0345836bd10956d7c9670dc3b8e3b . "./code.cg2js/cg-sample-file-menu.fasl")
(#x04ce8e5998c82b4061b8eec504155079 . "./code.cg2js/cg-scaling-stream.fasl")
(#x2260281b6fab9990ea1fcf9bdb2727a7 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x59bcaf730ff25e082425305cf7d59768 . "./code.cg2js/cg-scroll-bar.fasl")
(#x732b993c627e272662ba8db7017f50a1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xea08eeb088048ee5c5b65465ce2b587f . "./code.cg2js/cg-selected-object.fasl")
(#x705660df95a70e52872251435d3f2aa9 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xd5bac82e15d6a7e156cab0d03f3940a3 . "./code.cg2js/cg-split-bar.fasl")
(#xb3d735da18ee1e24d64d05c46278ff39 . "./code.cg2js/cg-static-text.fasl")
(#xcbe3a6498f9e57f85861306b09c8b0cc . "./code.cg2js/cg-status-bar.fasl")
(#xdc3215ab51bf102c4f3338fe28c09b1b . "./code.cg2js/cg-string-dialog.fasl")
(#x492c790b4dd17bcaddaa428e588089d0 . "./code.cg2js/cg-tab-control.fasl")
(#xf643b7e71ba29ed217869ae8653b60d0 . "./code.cg2js/cg-template-string.fasl")
(#x89a0777424c428607d898478ead846cb . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x0ac00aab2aa577f65c417416830009a5 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xd213bed334a8df591bbd8a1fcb3b6397 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xe9610b1aab5c4986400c27af243d3e95 . "./code.cg2js/cg-text-or-combo.fasl")
(#xf3d37ef33deace5ff08b171b9645940c . "./code.cg2js/cg-text-widget.fasl")
(#xc40b6ac97cc469a2b082f0b6d1ea3049 . "./code.cg2js/cg-timer.fasl")
(#x8c179bd6e2b3bec39691cbc5b6c38007 . "./code.cg2js/cg-toggling-widget.fasl")
(#x3d973a1b09e8359e047d8d4d1e06a7bd . "./code.cg2js/cg-toolbar.fasl")
(#x9ca237c4e984fd95698af9ae9835d27e . "./code.cg2js/cg-tooltip.fasl")
(#x65f8801d64096a951f5ffe39443fe2f6 . "./code.cg2js/cg-touch.fasl")
(#x4295e230d79333bd1955ef50f70050f4 . "./code.cg2js/cg-trackbar.fasl")
(#x26b7c4760432f035040d5f9b8ef66d2d . "./code.cg2js/cg-tray.fasl")
(#x1c60f0ab76150ea9baf272c8c2b2285f . "./code.cg2js/cg-up-down-control.fasl")
(#x9154b69284f33c0ebe32d56dca0166f4 . "./code.cg2js/cg-utility-dialog.fasl")
(#xa5ba22715a8bf38f8fefca41a424ca76 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x8f59ca9071c17ca2db7dfb7934749746 . "./code.cg2js/cg-web-browser.fasl")
(#xa79ae915fcbb817c9910208c07f2ffa3 . "./code.cg2js/cg-wrap-string.fasl")
(#x0488e3dc45d2dfbfefd6ece0fca3120a . "./code.cg2js/cg-yes-no-list.fasl")
(#x9ef4f932a3a14455c340affa1118b08f . "./code.cg2js/cg-yes-no-string.fasl")
(#xa1ed2a9fff44cc2e1e2307a854f8af8d . "./code.cg2js/cg.fasl")
(#x6d4d6365ab7642a3804777c8069df9b6 . "./code.idejs/ide-acache.fasl")
(#xe3bc9091706f8a0142c6b5f98920c501 . "./code.idejs/ide-apropos.fasl")
(#x3b20050f32ab3990de0d1093f97068eb . "./code.idejs/ide-base.fasl")
(#xfaeb91db2a9e2c4b4fc509aa3dbd748e . "./code.idejs/ide-class-browser.fasl")
(#x5d115771deeef483e7f658a7dfb3c572 . "./code.idejs/ide-class-grapher.fasl")
(#x3160d58d04586c14301e16315d56e610 . "./code.idejs/ide-clipboard.fasl")
(#x9337a88496a16d473819895cd435beb7 . "./code.idejs/ide-dev.fasl")
(#x4104be8ce5f5a2caee04692088a4a57b . "./code.idejs/ide-editor.fasl")
(#x652f4f220e51508e09d8181de897441c . "./code.idejs/ide-find-definitions.fasl")
(#x8285335e3fd65f4c8329d94a0c6f1101 . "./code.idejs/ide-find-in-files.fasl")
(#x29a41a9c4f568c17a4f5325ccbdb8835 . "./code.idejs/ide-form.fasl")
(#xad849374d398171be5f83a99a8708b4b . "./code.idejs/ide-grapher.fasl")
(#x0e1c2ce111e5a457db6d6dae01f9dfbf . "./code.idejs/ide-inspector.fasl")
(#xcca79544e60c01e8693cb66629bc5f50 . "./code.idejs/ide-intro.fasl")
(#xeac15714b6e27ffae22f9af695a3fb89 . "./code.idejs/ide-keyshort.fasl")
(#x658004544b0675ebe7529014f4b0a6b3 . "./code.idejs/ide-navigator.fasl")
(#xaace155691e5ee5fa8e351ea531cc221 . "./code.idejs/ide-package-list.fasl")
(#x62ea036c2fefb18e878ccb6b55a2594b . "./code.idejs/ide-patch-dialog.fasl")
(#x319d29762e6cc6523136a1aec8d5e613 . "./code.idejs/ide-process-browser.fasl")
(#x42e93f9693810c175ee6deeaf3e31837 . "./code.idejs/ide-project.fasl")
(#xcc7b8bd9505e305bd67adaeee5189bc7 . "./code.idejs/ide-trace.fasl")
(#xdb4f59ca8e056980c26eab136f8c0769 . "./code.idejs/ide-window-list.fasl")
(#xf73559dd0bffbe959871084285d00d39 . "./code.idejs/ide.fasl")
(#xdf73f3983b983ba8030e7ee4702b498a . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
